import React, { useState, createContext, useContext, PropsWithChildren } from 'react'

// 构造器函数，用于创建全局状态管理钩子
function createGlobalStateHook<T,G=undefined>(initialState?: T, getterSetter?: (state?: T)=>G) {
  const GlobalStateContext = createContext<T|undefined>(initialState)
  const GlobalStateUpdateContext = createContext<React.Dispatch<React.SetStateAction<T|undefined>>>(() => {})
  const GlobalStateGetterContext = createContext<G|undefined>(undefined)


  function useGlobalState() {
    return useContext(GlobalStateContext)
  }

  function useGlobalStateUpdate() {
    return useContext(GlobalStateUpdateContext)
  }

  function useGlobalStateGetter() {
    return useContext(GlobalStateGetterContext)
  }

  function GlobalStateProvider({ children }: PropsWithChildren) {
    const [globalState, setGlobalState] = useState(initialState)
    const getters = getterSetter?.(globalState)

    return (
      <GlobalStateContext.Provider value={globalState}>
        <GlobalStateUpdateContext.Provider value={setGlobalState}>
          <GlobalStateGetterContext.Provider value={getters}>
            {children}
          </GlobalStateGetterContext.Provider>
        </GlobalStateUpdateContext.Provider>
      </GlobalStateContext.Provider>
    )
  }

  return {
    useGlobalState,
    useGlobalStateUpdate,
    GlobalStateProvider,
    useGlobalStateGetter
  }
}

export default createGlobalStateHook
